/**************************************
*                                     *
*   Jeff Molofee's Basecode Example   *
*          nehe.gamedev.net           *
*                2001                 *
*                                     *
**************************************/

#include <windows.h>											// Header File For Windows
#include <gl\gl.h>												// Header File For The OpenGL32 Library
#include <gl\glu.h>												// Header File For The GLu32 Library
#include <gl\glaux.h>
#include <stdio.h>
#include <map>
#include "NeHeGL.h"
#include "Mesh.h"

#pragma comment( lib, "opengl32.lib" )							// Search For OpenGL32.lib While Linking
#pragma comment( lib, "glu32.lib" )								// Search For GLu32.lib While Linking
#pragma comment( lib, "glaux.lib" )	

#ifndef CDS_FULLSCREEN											// CDS_FULLSCREEN Is Not Defined By Some
#define CDS_FULLSCREEN 4										// Compilers. By Defining It This Way,
#endif															// We Can Avoid Errors

GL_Window*	g_window;
Keys*		g_keys;

// User Defined Variables
float		angle;												// Used To Rotate The Triangles
remMesh		gModel;
std::map<int, GLuint> Textures;
int			n, i, j;
GLfloat		White[4] = { 1.0f, 1.0f, 1.0f, 1.0f };

GLuint LoadTextureBMP(const char *Filename)
{
	GLuint lTex;
	FILE *File = NULL;
	if (!Filename) return -1;
	File = fopen(Filename, "r");
	
	if (!File)
	{
		return -1;
	}
	
	AUX_RGBImageRec *TextureImage[1];
	memset(TextureImage, 0, sizeof(void *)*1); 
	
	if (TextureImage[0] = auxDIBImageLoad(Filename))
	{
		glGenTextures(1, &lTex);
		glBindTexture(GL_TEXTURE_2D, lTex);
		glTexImage2D(GL_TEXTURE_2D, 0, 3, TextureImage[0]->sizeX, TextureImage[0]->sizeY, 0,
			GL_RGB, GL_UNSIGNED_BYTE, TextureImage[0]->data);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
/*
		gluBuild2DMipmaps(GL_TEXTURE_2D, 3, TextureImage[0]->sizeX, TextureImage[0]->sizeY,
			GL_RGB, GL_UNSIGNED_BYTE, TextureImage[0]->data);
*/
		
	}
	
	if (TextureImage[0])
	{
		if (TextureImage[0]->data) {
			free(TextureImage[0]->data);
		} 
		free(TextureImage[0]);
	}
	fclose(File);
	return lTex;
}

BOOL GL_Window::Initialize ()					// Any GL Init Code & User Initialiazation Goes Here
{
//	GLfloat l_amb[4] = { 1.0f, 1.0f, 0.7f, 0.7f };
	g_window	= this;
	g_keys		= this->keys;

	// Start Of User Initialization
	angle		= 0.0f;											// Set Starting Angle To Zero

	glClearColor (0.4f, 0.4f, 0.4f, 0.5f);						// Black Background
	glClearDepth (1.0f);										// Depth Buffer Setup
	glDepthFunc (GL_LEQUAL);									// The Type Of Depth Testing (Less Or Equal)
	glEnable (GL_DEPTH_TEST);									// Enable Depth Testing
	glShadeModel (GL_SMOOTH);									// Select Smooth Shading
	glEnable(GL_LIGHT0);
	glEnable(GL_LIGHTING);
//	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, l_amb);
	glHint (GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);			// Set Perspective Calculations To Most Accurate

	// Load model from MMM file;
	gModel.LoadFromFile("Test.MMM");
	for (n = 0; n < gModel.Model.size(); n++)
		if (gModel.Model[n].TextureName != "")
		{
			Textures[n] = LoadTextureBMP(gModel.Model[n].TextureName.data());
		}

	return TRUE;												// Return TRUE (Initialization Successful)
}


void GL_Window::Deinitialize (void)										// Any User DeInitialization Goes Here
{
}

void Update (DWORD milliseconds)								// Perform Motion Updates Here
{
	angle += (double)milliseconds / 50.0f;

	if (g_keys->keyDown [VK_ESCAPE] == TRUE)                                        // Is ESC Being Pressed?
	{
		g_window->TerminateApplication ();                                              // Terminate The Program
	}
	
	if (g_keys->keyDown [VK_F1] == TRUE)                                            // Is F1 Being Pressed?
	{
		g_window->ToggleFullscreen ();                                                  // Toggle Fullscreen Mode
	}
}

void GL_Window::Draw (void)
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    glLoadIdentity();
	glTranslatef(0.0f, -100.0f, -200.0f);
	glRotatef(angle, 0.0f, 1.0f, 0.0f);
	// Set Material
	for (n = 0; n < gModel.Model.size(); n++)
	{
		if (gModel.Model[n].WithMtl)
		{
			glMaterialfv(GL_FRONT, GL_AMBIENT, &gModel.Model[n].mtlAmbient.C[0]);
			glMaterialfv(GL_FRONT, GL_DIFFUSE, &gModel.Model[n].mtlDiffuse.C[0]);
			glMaterialfv(GL_FRONT, GL_SPECULAR, &gModel.Model[n].mtlSpecular.C[0]);
			glMaterialf(GL_FRONT, GL_SHININESS, gModel.Model[n].mtlSpecular.C[3] * 100);
			
			if (gModel.Model[n].TextureName != "") 
			{
				glEnable(GL_TEXTURE_2D);
				glBindTexture(GL_TEXTURE_2D, Textures[n]);
			}
			else
				glDisable(GL_TEXTURE_2D);
		}
		else
			glColor4fv(&White[0]);
		
		glBegin(GL_TRIANGLES);
		for (i = 0; i < gModel.Model[n].MeshFaces.size(); i++)
			for (j = 0; j < 3; j++)
			{
				if (gModel.Model[n].MeshTVerts.size() != 0) 
					glTexCoord2fv(&gModel.Model[n].MeshTVerts[gModel.Model[n].MeshFaces[i].VertIndex[j+3]].vert[0]);
				glNormal3fv(&gModel.Model[n].MeshVerts[gModel.Model[n].MeshFaces[i].VertIndex[j]].normal[0]);
				glVertex3fv(&gModel.Model[n].MeshVerts[gModel.Model[n].MeshFaces[i].VertIndex[j]].vert[0]);
			}
			glEnd();
	}
}
