//===========================================================
//  Filename: remMesh.cpp
//      3D object incapsulation class. Load, and store data
//      Copyright 2003
//      Created by Roman E. Marchenko, vortex@library.ntu-kpi.kiev.ua
//      $Id$
//===========================================================

#include <fstream.h>
#include "ExpDefines.h"
#include "Mesh.h"

remMesh::remMesh()
{
}

remMesh::~remMesh()
{

}

bool remMesh::LoadFromFile(const rStr &Filename)
{
	streampos NextNode = 0, Tmp, Face_p, Vertex_p;
	SingleGeomNode N;
	int i, Hdr = 0;
	char Ch;

	ifstream inFile(Filename.c_str(), ios::in | ios::binary | ios::nocreate);
	// if file open error
	if (!inFile) return false;
	// Else read file and save data to vector MeshVerts
	while (inFile.good())
	{
		N.WithMtl = false;
		N.WithTexture = false;
		N.TextureName = "";
		// read num faces and vertexes
		inFile.seekg(NextNode + 2);
		inFile.read((char *)&Tmp, 4);
		N.MeshVerts.resize(Tmp);
		inFile.read((char *)&Tmp, 4);
		N.MeshTVerts.resize(Tmp);
		inFile.read((char *)&Tmp, 4);
		N.MeshFaces.resize(Tmp);

		// process with data
		inFile.read((char *)&NextNode, 4);
		inFile.seekg(inFile.tellg() + 2);
		// Reed material data
		inFile.read((char *)&Vertex_p, 4);
		// Check if material present
		inFile.read((char *)&Hdr, 2);
		inFile.seekg(inFile.tellg() - 2);
		if (Hdr != ID_VERTEX_HEADER)
		{
			N.WithMtl = true;
			// read material components
			inFile.read((char *)&N.mtlAmbient.C[0], sizeof(float) * 3);
			inFile.read((char *)&N.mtlDiffuse.C[0], sizeof(float) * 3);
			inFile.read((char *)&N.mtlSpecular.C[0], sizeof(float) * 3);
			// read Shininess
			inFile.read((char *)&N.mtlSpecular.C[3], sizeof(float));
			// read filename
			do {
				inFile >> Ch;
				N.TextureName += Ch;
			} while(Ch != 0);
			N.TextureName.resize(N.TextureName.size() - 1);
		}
		// Read Vertex data.
		inFile.seekg(inFile.tellg() + 2);
		inFile.read((char *)&Face_p, 4);
		while (inFile.good() && (inFile.tellg() < NextNode))
		{
			for (i = 0; i < N.MeshVerts.size(); i++)
			{
				inFile.read((char *)&N.MeshVerts[i].vert[0], sizeof(float) * 3);
				inFile.read((char *)&N.MeshVerts[i].normal[0], sizeof(float) * 3);
			}
			// Read Texture vertex data
			inFile.read((char *)&Tmp, 2);
			if (Tmp == ID_TVERTEX_HEADER)
			{
				for (i = 0; i < N.MeshTVerts.size(); i++)
				{
					inFile.read((char *)&N.MeshTVerts[i].vert[0], sizeof(float) * 2);
				}
			}
			else
				inFile.seekg(inFile.tellg() - 2);

			// Skip Face data block header
			inFile.read((char *)&Tmp, 2);
			// Read face data
			for (i = 0; i < N.MeshFaces.size(); i++)
			{
				if (N.MeshTVerts.size() != 0)
					inFile.read((char *)&N.MeshFaces[i].VertIndex[0], sizeof(float) * 6);
				else
					inFile.read((char *)&N.MeshFaces[i].VertIndex[0], sizeof(float) * 3);
			}
			inFile.ignore(1);
		}
		Model.push_back(N);
	}
	return true;
}
