//===========================================================
//  Filename: remMesh.h
//      3D object incapsulation class. Load, and store data
//      Copyright 2003
//      Created by Roman E. Marchenko, vortex@library.ntu-kpi.kiev.ua
//      $Id$
//===========================================================

#ifndef _REM_MESH_H_
#define _REM_MESH_H_

#include <vector>
#include <string>

typedef std::string rStr;

struct rColor4f
{
	union
	{
		struct RGBColor
		{
			float r, g, b, a;
		} RGB;
		float C[4];
	};
	
	rColor4f& operator = (const rColor4f &right)
	{
		C[0] = right.C[0];
		C[1] = right.C[1];
		C[2] = right.C[2];
		C[3] = right.C[3];
		return *this;
	}
	
	bool operator == (const rColor4f &right)
	{
		return ((C[0] = right.C[0]) &&
			(C[1] = right.C[1]) &&
			(C[3] = right.C[3]) &&
			(C[2] = right.C[2]));
	}
	
	bool operator != (const rColor4f &right)
	{
		return ((C[0] != right.C[0]) ||
			(C[1] != right.C[1]) ||
			(C[3] != right.C[3]) ||
			(C[2] != right.C[2]));
	}
	
	// constructors
	rColor4f(float r, float g, float b, float a)
	{
		RGB.r = r; RGB.g = g; RGB.b = b; RGB.a = a;
	}
	
	rColor4f()
	{
		RGB.r = 0; RGB.g = 0; RGB.b = 0; RGB.a = 0;
	}
	
};

typedef struct 
{
	float vert[3], normal[3];
} remVertexData;

typedef struct 
{
	float vert[3];
} remTVertexData;

// Face description struct. First three indices are geom verts, 
// another three are texture verts
typedef struct 
{
    int VertIndex[6];
} remFaceData;

typedef struct 
{
	std::vector<remVertexData>      MeshVerts;
	std::vector<remTVertexData>     MeshTVerts;
	std::vector<remFaceData>        MeshFaces;
	rColor4f                        mtlAmbient, mtlDiffuse, mtlSpecular;
	void                            *ModelTex;
	rStr							TextureName;
	// Info members
	bool                            WithTexture, WithMtl;
} SingleGeomNode;

class remMesh  
{
public:
	// data storage
	std::vector<SingleGeomNode> Model;
	// Loading routine
	bool LoadFromFile(const rStr &Filename);

	remMesh();
	virtual ~remMesh();
};

#endif
