//===========================================================
//  Filename: MyExp.cpp
//      3D studio MAX exporter plugin
//      Copyright 2003
//      Created by Roman E. Marchenko, vortex@library.ntu-kpi.kiev.ua
//      $Id$
//===========================================================

#include "MyExp.h"

// Global variables

static SceneSaver	MyTreeEnum;
static ofstream		fFile;
Interface			*ip;

MyExp::MyExp()
{
}

MyExp::~MyExp()
{
}

int MyExp::DoExport(const TCHAR *name, ExpInterface *ei,
					Interface *i, BOOL suppressPromts, DWORD options)
{
	fFile.open(name, ios::out | ios::binary);
	// Globalize i var
	ip = i;
	// Go Through all scene nodes
	ei->theScene->EnumTree(&MyTreeEnum);

	fFile.close();
	return 1;
}

BOOL MyExp::SupportsOptions(int ext, DWORD options)
{
	return FALSE;
}

//-----------------------------------------------------------
//    Scene saver class
//-----------------------------------------------------------

int SceneSaver::callback(INode *node)
{
	ProcNode(node);
	return TREE_CONTINUE;
}

// All node process here
void SceneSaver::ProcNode(INode *node)
{
	int			numF, numV, numTV, CurrHeader, zero = 0, debug = 0xAA, Del;
	streampos	NextNode, VertexPointer, FacePointer, TmpPos;
	// Get TriObject from node
	TriObject *TObj;
	TObj = GetTriObjFromNode(node, Del);
	if (!TObj) return;
	numF = TObj->mesh.numFaces;
	numV = TObj->mesh.numVerts;
	numTV = TObj->mesh.numTVerts;
	CurrHeader = ID_NODE_HEADER;
	// Write node header and total count of vertexes and faces
	fFile.write((char *)&CurrHeader, 2);
	fFile.write((char *)&numV, sizeof(int));
	fFile.write((char *)&numTV, sizeof(int));
	fFile.write((char *)&numF, sizeof(int));
	// We don't know how long node record is. 
	// Save stream pos, and temporary fill 4 bytes by zeroes.
	NextNode = fFile.tellp();
	fFile.write((char *)&zero, 4);
	// Write Material header
	CurrHeader = ID_MTL_HEADER;
	fFile.write((char *)&CurrHeader, 2);
	// Remember current position as place for address for mesh section
	VertexPointer = fFile.tellp();
	fFile.write((char *)&zero, 4);
	// Export material associated with node
	ExportMaterial(node);
	// Write current pos as begin of Vertex section of our file
	TmpPos = fFile.tellp();
	fFile.seekp(VertexPointer);
	fFile.write((char *)&TmpPos, 4);
	fFile.seekp(TmpPos);
	// Begin write geometry data
	// First of all dump 2 byte mesh header
	CurrHeader = ID_VERTEX_HEADER;
	fFile.write((char *)&CurrHeader, 2);
	// Then save place for Face data pointer
	FacePointer = fFile.tellp();
	fFile.write((char *)&zero, 4);
	// Then pass through all node's verts and put into file all data:
	// verex coordinates, vertex mapping UV coordinates and normal vector
	// but first get TM for this node
	ExportVerts(TObj, node->GetObjTMAfterWSM(ip->GetTime()));
	// Now save this pos as begin face block
	TmpPos = fFile.tellp();
	fFile.seekp(FacePointer);
	fFile.write((char *)&TmpPos, 4);
	fFile.seekp(TmpPos);
	// Now export all Face data
	// First write Face data header
	CurrHeader = ID_FACE_HEADER;
	fFile.write((char *)&CurrHeader, 2);
	// Write all facees data
	ExportFaces(TObj);
	// Now put this pos as begin of new node
	TmpPos = fFile.tellp();
	fFile.seekp(NextNode);
	fFile.write((char *)&TmpPos, 4);
	fFile.seekp(TmpPos);
	// Go to the next node if anyone exists
}

// One has one material. Save it here
void SceneSaver::ExportMaterial(INode *node)
{
	Color		Cl;
	float		Shininess;
	char		zero = 0;
	// Get material assigned to node
	Mtl *m = node->GetMtl();
	if (!m) return;
	// See if it's a standart material
	if (m->ClassID() != Class_ID(DMTL_CLASS_ID, 0)) return;

	StdMat *mStd = (StdMat *)m;
	// Get Ambient component
	Cl = mStd->GetAmbient(ip->GetTime());
	fFile.write((char *)&Cl.r, sizeof(float));
	fFile.write((char *)&Cl.g, sizeof(float));
	fFile.write((char *)&Cl.b, sizeof(float));
	// Get Diffuse component
	Cl = mStd->GetDiffuse(ip->GetTime());
	fFile.write((char *)&Cl.r, sizeof(float));
	fFile.write((char *)&Cl.g, sizeof(float));
	fFile.write((char *)&Cl.b, sizeof(float));
	// Get Specular component
	Cl = mStd->GetSpecular(ip->GetTime());
	fFile.write((char *)&Cl.r, sizeof(float));
	fFile.write((char *)&Cl.g, sizeof(float));
	fFile.write((char *)&Cl.b, sizeof(float));
	// Get shininess
	Shininess = mStd->GetShininess(ip->GetTime()) + mStd->GetShinStr(ip->GetTime());
	fFile.write((char *)&Shininess, sizeof(float));
	// Get Diffuse texture name
	Texmap *tmap = m->GetSubTexmap(ID_DI);
	if (!tmap) 
	{
		fFile.write(&zero, 1);
		return;
	}
	if (tmap->ClassID() != Class_ID(BMTEX_CLASS_ID, 0))
	{
		fFile.write(&zero, 1);
		return;
	}
	// If bitmap exists
	BitmapTex *bmt = (BitmapTex *)tmap;
	// Write name of bitmap to file;
	
	fFile << ExtractFileName(bmt->GetMapName()).data();
	fFile.write(&zero, 1);
}

void SceneSaver::ExportVerts(TriObject *TObj, Matrix3 tm)
{
	int i, Hdr;
	Point3	nCoord;
	DWORD zero = 0;

	// Go through all faces in mesh and save it params to file
	TObj->mesh.buildNormals();
	// Write to file all verex coordinates and it's normals
	for (i = 0; i < TObj->mesh.numVerts; i++)
	{
		Point3 v = tm * TObj->mesh.verts[i];
		fFile.write((char *)&v.x, sizeof(DWORD));
		fFile.write((char *)&v.z, sizeof(DWORD));
		fFile.write((char *)&v.y, sizeof(DWORD));
		// Write normals (x, y, z)
		nCoord = TObj->mesh.getNormal(i);
		fFile.write((char *)&nCoord.x, sizeof(float));
		fFile.write((char *)&nCoord.z, sizeof(float));
		fFile.write((char *)&nCoord.y, sizeof(float));
	}
	// Write Texture coordinate data (U and V) if present
	if (TObj->mesh.numTVerts != 0)
	{
		// Write 2 byte texture data section header
		Hdr = ID_TVERTEX_HEADER;
		fFile.write((char *)&Hdr, 2);
		// Loop through all texture vertexes
		for (i = 0; i < TObj->mesh.numTVerts; i++)
		{
			fFile.write((char *)&TObj->mesh.tVerts[i].x, sizeof(DWORD));
			fFile.write((char *)&TObj->mesh.tVerts[i].y, sizeof(DWORD));
		}
	}
}

void SceneSaver::ExportFaces(TriObject *TObj)
{
	int		i;
	// loop through all faces and dump data to file
	for (i = 0; i < TObj->mesh.numFaces; i++)
	{
		fFile.write((char *)&TObj->mesh.faces[i].v[0], sizeof(DWORD) * 3);
		if (TObj->mesh.numTVerts != 0)
			fFile.write((char *)&TObj->mesh.tvFace[i].t[0], sizeof(DWORD) * 3);
	}
}

//-----------------------------------------------------------
//    Standalone functions
//-----------------------------------------------------------

TriObject *GetTriObjFromNode(INode *node, int &deleteIt)
{
	deleteIt = FALSE;
	Object *obj = node->EvalWorldState(ip->GetTime()).obj;
	if (obj->CanConvertToType(Class_ID(TRIOBJ_CLASS_ID,0)))
	{
		TriObject *tri = (TriObject *) obj->ConvertToType(ip->GetTime(), 
			Class_ID(TRIOBJ_CLASS_ID, 0));
		if (obj != tri) deleteIt = TRUE;
		return tri;
	}
	else return NULL;
}

remString ExtractFileName(remString filename)
{
        if (filename.size() == 0) return "";

        int i = filename.size();
        remString buf;
        while((filename[i] != '\\') && (i > 0))
        {
                buf = filename[i--] + buf;
        }
        return buf;
}